@@ -4,7 +4,7 @@ from .lp import LinProg
 import numpy as np
 import pandas as pd
 import scipy.sparse as sp
-from collections import Iterable
+from collections.abc import Iterable
 
 
 class Model(LPModel):
@@ -12,9 +12,9 @@ class Model(LPModel):
     The Model class creates an SOCP model object
     """
 
-    def __init__(self, nobj=False, mtype='R', name=None):
+    def __init__(self, nobj=False, mtype='R', name=None, top=None):
 
-        super().__init__(nobj, mtype, name)
+        super().__init__(nobj, mtype, name, top)
         self.cvx_constr = []
         self.cone_constr = []
         # self.avar_indices = []
@@ -43,8 +43,6 @@ class Model(LPModel):
                 super().st(constr)
             elif constr.xtype in 'ESQ':
                 self.cvx_constr.append(constr)
-            else:
-                raise TypeError('Incorrect constraint type.')
         elif isinstance(constr, ConeConstr):
             if constr.model is not self:
                 raise ValueError('Constraints are not defined for this model.')
@@ -55,7 +53,7 @@ class Model(LPModel):
         self.pupdate = True
         self.dupdate = True
 
-    def do_math(self, primal=True, refresh=True, obj=False):
+    def do_math(self, primal=True, refresh=True, obj=True):
 
         if primal:
             if self.primal is not None and not self.pupdate:
@@ -68,8 +66,8 @@ class Model(LPModel):
                 self.last = self.vars[-1].first + self.vars[-1].size
 
             more_cvx = []
-            if self.obj:
-                obj_constr = (self.vars[0] >= self.sign * self.obj)
+            if self.obj is not None:
+                obj_constr = (self.vars[0] - self.sign * self.obj >= 0)
                 if isinstance(obj_constr, CvxConstr):
                     more_cvx.append(obj_constr)
 
@@ -77,9 +75,9 @@ class Model(LPModel):
             for constr in self.cvx_constr + more_cvx:
                 if constr.xtype == 'E':
                     aux_left = self.dvar(constr.affine_in.shape, aux=True)
-                    aux_right = self.dvar(constr.affine_out.shape, aux=True)
+                    aux_right = self.dvar(1, aux=True)
                     self.aux_constr.append(constr.affine_in - aux_left == 0)
-                    self.aux_constr.append(constr.affine_out + aux_right == 0)
+                    self.aux_constr.append(constr.affine_out + aux_right <= 0)
                     bounds = (aux_right >= 0)
                     if isinstance(bounds, Bounds):
                         self.aux_bounds.append(bounds)
@@ -106,14 +104,18 @@ class Model(LPModel):
                         qmat.append([aux3.first + i] +
                                     [aux1.first + i, aux2.first + i])
                 elif constr.xtype == 'Q':
-                    aux1 = self.dvar(constr.affine_out.shape, aux=True)
+                    # aux1 = self.dvar(constr.affine_out.shape, aux=True)
+                    aux1 = self.dvar(1, aux=True)
                     aux2 = self.dvar(constr.affine_in.shape, aux=True)
-                    aux3 = self.dvar(constr.affine_out.shape, aux=True)
-                    self.aux_constr.append(aux1 - 0.5 * (1+constr.affine_out)
+                    # aux3 = self.dvar(constr.affine_out.shape, aux=True)
+                    aux3 = self.dvar(1, aux=True)
+                    aux4 = self.dvar(1, aux=True)
+                    self.aux_constr.append(aux1 - 0.5 * (1-aux4)
                                            == 0)
                     self.aux_constr.append(aux2 - constr.affine_in == 0)
-                    self.aux_constr.append(aux3 - 0.5 * (1-constr.affine_out)
+                    self.aux_constr.append(aux3 - 0.5 * (1+aux4)
                                            == 0)
+                    self.aux_constr.append(aux4 + constr.affine_out <= 0)
                     bounds = (aux3 >= 0)
                     if isinstance(bounds, Bounds):
                         self.aux_bounds.append(bounds)
@@ -150,28 +152,59 @@ class Model(LPModel):
                 return formula
 
             eye_indices = [item for inner in primal.qmat for item in inner]
-            lin_indices = [ind for ind in range(primal.linear.shape[1])
-                           if ind not in eye_indices]
-            linear = dual_lp.linear[lin_indices, :]
-            const = dual_lp.const[lin_indices]
-            sense = dual_lp.sense[lin_indices]
-            obj = dual_lp.obj
-            vtype = dual_lp.vtype
-            ub = dual_lp.ub
-            lb = dual_lp.lb
-            qmat = []
-            for qc in primal.qmat:
-                lbz_index = dual_lp.linear[qc[0], :].indices
-                ub[lbz_index] = - lb[lbz_index]
-                lb[lbz_index] = 0
-                linear[:, lbz_index] = - linear[:, lbz_index]
-                obj[lbz_index] = - obj[lbz_index]
-                # ub[dual_lp.linear[qc[0], :].indices] = 0
-                ################## To update in the future
-                qmat.append(list(dual_lp.linear[qc, :].indices))
+            eye_block = dual_lp.linear[eye_indices, :]
+            if len(eye_block.data) + 1 == len(eye_block.indptr):
+                lin_indices = [ind for ind in range(primal.linear.shape[1])
+                               if ind not in eye_indices]
+                linear = dual_lp.linear[lin_indices, :]
+                const = dual_lp.const[lin_indices]
+                sense = dual_lp.sense[lin_indices]
+                obj = dual_lp.obj
+                vtype = dual_lp.vtype
+                ub = dual_lp.ub
+                lb = dual_lp.lb
+                qmat = []
+                for qc in primal.qmat:
+                    lbz_index = dual_lp.linear[qc[0], :].indices
+                    ub[lbz_index] = - lb[lbz_index]
+                    lb[lbz_index] = 0
+                    linear[:, lbz_index] = - linear[:, lbz_index]
+                    obj[lbz_index] = - obj[lbz_index]
+                    qmat.append(list(dual_lp.linear[qc, :].indices))
 
-            formula = SOCProg(linear, const, sense,
-                              vtype, ub, lb, qmat, obj)
+                formula = SOCProg(linear, const, sense,
+                                  vtype, ub, lb, qmat, obj)
+            else:
+                linear = dual_lp.linear
+                num_constr = dual_lp.linear.shape[1]
+                extra_nvar = len(eye_indices)
+                extra_block = sp.csr_matrix((np.ones(extra_nvar),
+                                             (eye_indices, np.arange(extra_nvar))),
+                                            shape=(linear.shape[0], extra_nvar))
+                linear = sp.csr_matrix(sp.hstack((linear, extra_block)))
+                const = dual_lp.const
+                sense = dual_lp.sense
+
+                obj = np.concatenate((dual_lp.obj, np.zeros(extra_nvar)))
+                vtype = np.concatenate((dual_lp.vtype, np.array(['C']*extra_nvar)))
+
+                ub = np.concatenate((dual_lp.ub, np.ones(extra_nvar)*np.infty))
+                extra_lb = - np.ones(extra_nvar)*np.infty
+                ind_pos = 0
+                for qc in primal.qmat:
+                    extra_lb[ind_pos] = 0
+                    ind_pos += len(qc)
+                lb = np.concatenate((dual_lp.lb, extra_lb))
+
+                qmat = []
+                ind_pos = 0
+                for qc in primal.qmat:
+                    qmat.append([ind_pos+num_constr] +
+                                [k+ind_pos+num_constr for k in range(1, len(qc))])
+                    ind_pos += len(qc)
+
+                formula = SOCProg(linear, const, sense,
+                                  vtype, ub, lb, qmat, obj)
 
             self.dual = formula
             self.dupdate = False
@@ -189,11 +222,11 @@ class SOCProg(LinProg):
         super().__init__(linear, const, sense, vtype, ub, lb, obj)
         self.qmat = qmat
 
-    def __repr__(self, header=False):
+    def __repr__(self):
 
         qmat = self.qmat
         string = 'Second order cone program object:\n'
-        string += super().__repr__(header=header)
+        string += super().__repr__()
         string += '---------------------------------------------\n'
         string += 'Number of SOC constraints:    {0}\n'.format(len(qmat))
 
@@ -249,3 +282,17 @@ class SOCProg(LinProg):
         table = pd.concat([table, ub, lb, vtype], axis=0)
 
         return table.fillna('-')
+
+    def lp_export(self):
+
+        string = super().lp_export()
+        index_st = string.find('Subject To')
+        s1 = string[:index_st+11]
+        s2 = string[index_st+11:]
+        sq = ''
+        for i, qc in enumerate(self.qmat):
+            sq += ' q{}: [ '.format(i+1)
+            sq += ' + '.join(['x{} ^2'.format(j+1) for j in qc[1:]])
+            sq += ' - x{} ^2 ] <= 0\n'.format(qc[0]+1)
+
+        return s1 + sq + s2
